#include "stdafx.h"
#include "AxControl.h"
#include <crtdbg.h>

class CControlContainer:public IOleClientSite,public IOleInPlaceSite
{
	HWND m_hWnd;
	ULONG m_refCnt;
public:
	CControlContainer(HWND hWnd)
	{
		m_hWnd = hWnd;
		m_refCnt = 1;
	}
	~CControlContainer()
	{
		_ASSERT(m_refCnt == 0);
	}
	//IUnknown
	STDMETHOD_(ULONG,AddRef())
	{
		return ++m_refCnt;
	}
	STDMETHOD_(ULONG,Release())
	{
		if (--m_refCnt == 0)
		{
			delete this;
			return 0;
		}
		return m_refCnt;
	}
	STDMETHOD(QueryInterface(REFIID riid,void **ppvObject))
	{
		*ppvObject = NULL;
		if ( IID_IUnknown == riid)
		{
			*ppvObject = (IUnknown*)(IOleInPlaceSite*)this;
		}
		else if (IID_IOleClientSite == riid)
		{
			*ppvObject = (IOleClientSite*)this;
		}
		else if (IID_IOleWindow == riid)
		{
			*ppvObject = (IOleWindow*)this;
		}
		else if (IID_IOleInPlaceSite == riid)
		{
			*ppvObject = (IOleInPlaceSite*)this;
		}
		else
			return E_NOINTERFACE;
		AddRef();
		return S_OK;
	}
	//IOleControlSite
	STDMETHOD(SaveObject())
	{
		return E_NOTIMPL;
	}
	STDMETHOD(GetMoniker(DWORD,DWORD,IMoniker**))
	{
		return E_NOTIMPL;
	}
	STDMETHOD(GetContainer(IOleContainer **ppContainer))
	{
		return E_NOINTERFACE;
	}
	STDMETHOD(ShowObject())
	{
		return S_OK;
	}
	STDMETHOD(OnShowWindow(BOOL bShow))
	{
		return S_OK;
	}
	STDMETHOD(RequestNewObjectLayout())
	{
		return E_NOTIMPL;
	}
	//IOleWindow
	STDMETHOD(GetWindow(HWND * pHwnd))
	{
		*pHwnd = m_hWnd;
		return S_OK;
	}
	STDMETHOD(ContextSensitiveHelp(BOOL bEnterMode))
	{
		return S_OK;
	}
	//IOleInPlaceSite
	STDMETHOD(CanInPlaceActivate())
	{
		return S_OK;
	}
	STDMETHOD(OnInPlaceActivate())
	{
		return S_OK;
	}
	STDMETHOD(OnUIActivate())
	{
		return S_OK;
	}
	STDMETHOD(GetWindowContext(/* [out] */ IOleInPlaceFrame **ppFrame,
		/* [out] */ IOleInPlaceUIWindow **ppDoc,
		/* [out] */ LPRECT lprcPosRect,
		/* [out] */ LPRECT lprcClipRect,
		/* [out][in] */ LPOLEINPLACEFRAMEINFO lpFrameInfo))
	{
		return E_NOTIMPL;
	}
	STDMETHOD(Scroll(SIZE scrollSize))
	{
		return S_OK;
	}
	STDMETHOD(OnUIDeactivate(BOOL bUndoable))
	{
		return S_OK;
	}
	STDMETHOD(OnInPlaceDeactivate())
	{
		return S_OK;
	}
	STDMETHOD(DiscardUndoState())
	{
		return S_OK;
	}
	STDMETHOD(DeactivateAndUndo())
	{
		return S_OK;
	}
	STDMETHOD(OnPosRectChange(LPCRECT lprcPosRect))
	{
		return S_OK;
	}
};


HRESULT CreateAxControl(HWND hWnd,const wchar_t * ProgId,IUnknown ** ppControlUnknown,IUnknown ** ppContainerUnknown)
{
	HRESULT hr;
	CLSID cls;
	IOleObject * pObject = NULL;
	CControlContainer * pContainer = NULL;
	hr = CLSIDFromProgID(ProgId,&cls);
	if (FAILED(hr)) goto exit1;
	
	hr = CoCreateInstance(cls,NULL,CLSCTX_INPROC_SERVER|CLSCTX_INPROC_HANDLER|CLSCTX_LOCAL_SERVER,IID_IOleObject,(void**)&pObject);
	if (FAILED(hr)) goto exit1;

	pContainer = new CControlContainer(hWnd);
	hr = pObject->SetClientSite(pContainer);
	if (FAILED(hr)) goto exit1;
	hr = pObject->DoVerb(OLEIVERB_SHOW,0,pContainer,0,hWnd,0);
	if (FAILED(hr)) goto exit1;

    pObject->QueryInterface(IID_IUnknown,(void**)ppControlUnknown);
	pContainer->QueryInterface(IID_IUnknown,(void**)ppContainerUnknown);
	hr = S_OK;
exit1:
	if (pObject) pObject->Release();
	if (pContainer) pContainer->Release();
	return hr;
}

HRESULT DispSetProperty(IUnknown * pUnk,const wchar_t * prop,VARIANT * pVal)
{
	IDispatch * pDisp = NULL;
	HRESULT hr;
	DISPID propId;
	const wchar_t * ppNames[] = {prop};
	DISPPARAMS dispparams;
	DISPID paramDispId;
	hr = pUnk->QueryInterface(IID_IDispatch,(void**)&pDisp);
	if (FAILED(hr)) return hr;
	//Query prop id
	hr = pDisp->GetIDsOfNames(IID_NULL,(LPOLESTR*)ppNames,1,LANG_SYSTEM_DEFAULT,&propId);
	if (FAILED(hr)) goto exit1;
	//call
	paramDispId = DISPID_PROPERTYPUT;
	dispparams.cArgs = 1;
	dispparams.cNamedArgs = 1;
	dispparams.rgdispidNamedArgs =&paramDispId;
	dispparams.rgvarg = pVal;
	hr = pDisp->Invoke(propId,IID_NULL,LOCALE_SYSTEM_DEFAULT,DISPATCH_PROPERTYPUT,&dispparams,NULL,NULL,NULL);
exit1:
	if (pDisp) pDisp->Release();
	return hr;

}